@@ -3,12 +3,23 @@ from __future__ import print_function, division, absolute_import
 from functools import partial
 
 from .compression import compressions, default_compression
-from .core import (dumps, loads, maybe_compress, decompress, msgpack)
+from .core import dumps, loads, maybe_compress, decompress, msgpack
+from .cuda import cuda_serialize, cuda_deserialize
 from .serialize import (
-    serialize, deserialize, nested_deserialize, Serialize, Serialized,
-    to_serialize, register_serialization, dask_serialize, dask_deserialize,
-    serialize_bytes, deserialize_bytes, serialize_bytelist,
-    register_serialization_family, register_generic,
+    serialize,
+    deserialize,
+    nested_deserialize,
+    Serialize,
+    Serialized,
+    to_serialize,
+    register_serialization,
+    dask_serialize,
+    dask_deserialize,
+    serialize_bytes,
+    deserialize_bytes,
+    serialize_bytelist,
+    register_serialization_family,
+    register_generic,
 )
 
 from ..utils import ignoring
@@ -50,16 +61,27 @@ def _register_arrow():
     from . import arrow
 
 
-@dask_serialize.register_lazy("sklearn")
-@dask_deserialize.register_lazy("sklearn")
-def _register_sklearn():
-    import sklearn.base
-    register_generic(sklearn.base.BaseEstimator)
-
-
 @dask_serialize.register_lazy("torch")
 @dask_deserialize.register_lazy("torch")
 @dask_serialize.register_lazy("torchvision")
 @dask_deserialize.register_lazy("torchvision")
 def _register_torch():
     from . import torch
+
+
+@cuda_serialize.register_lazy("cupy")
+@cuda_deserialize.register_lazy("cupy")
+def _register_cupy():
+    from . import cupy
+
+
+@cuda_serialize.register_lazy("numba")
+@cuda_deserialize.register_lazy("numba")
+def _register_numba():
+    from . import numba
+
+
+@cuda_serialize.register_lazy("cudf")
+@cuda_deserialize.register_lazy("cudf")
+def _register_cudf():
+    from . import cudf
