@@ -11,9 +11,8 @@ except ImportError:
     from toolz import reduce
 
 from .compression import compressions, maybe_compress, decompress
-from .serialize import (serialize, deserialize, Serialize, Serialized,
-                        extract_serialize)
-from .utils import frame_split_size, merge_frames
+from .serialize import serialize, deserialize, Serialize, Serialized, extract_serialize
+from .utils import frame_split_size, merge_frames, msgpack_opts
 from ..utils import nbytes
 
 _deserialize = deserialize
@@ -22,7 +21,7 @@ _deserialize = deserialize
 logger = logging.getLogger(__name__)
 
 
-def dumps(msg, serializers=None, on_error='message'):
+def dumps(msg, serializers=None, on_error="message", context=None):
     """ Transform Python message to bytestream suitable for communication """
     try:
         data = {}
@@ -34,55 +33,60 @@ def dumps(msg, serializers=None, on_error='message'):
         if not data:  # fast path without serialized data
             return small_header, small_payload
 
-        pre = {key: (value.header, value.frames)
-               for key, value in data.items()
-               if type(value) is Serialized}
+        pre = {
+            key: (value.header, value.frames)
+            for key, value in data.items()
+            if type(value) is Serialized
+        }
 
-        data = {key: serialize(value.data,
-                               serializers=serializers,
-                               on_error=on_error)
-                for key, value in data.items()
-                if type(value) is Serialize}
+        data = {
+            key: serialize(
+                value.data, serializers=serializers, on_error=on_error, context=context
+            )
+            for key, value in data.items()
+            if type(value) is Serialize
+        }
 
-        header = {'headers': {},
-                  'keys': [],
-                  'bytestrings': list(bytestrings)}
+        header = {"headers": {}, "keys": [], "bytestrings": list(bytestrings)}
 
         out_frames = []
 
         for key, (head, frames) in data.items():
-            if 'lengths' not in head:
-                head['lengths'] = tuple(map(nbytes, frames))
-            if 'compression' not in head:
+            if "lengths" not in head:
+                head["lengths"] = tuple(map(nbytes, frames))
+            if "compression" not in head:
                 frames = frame_split_size(frames)
                 if frames:
                     compression, frames = zip(*map(maybe_compress, frames))
                 else:
                     compression = []
-                head['compression'] = compression
-            head['count'] = len(frames)
-            header['headers'][key] = head
-            header['keys'].append(key)
+                head["compression"] = compression
+            head["count"] = len(frames)
+            header["headers"][key] = head
+            header["keys"].append(key)
             out_frames.extend(frames)
 
         for key, (head, frames) in pre.items():
-            if 'lengths' not in head:
-                head['lengths'] = tuple(map(nbytes, frames))
-            head['count'] = len(frames)
-            header['headers'][key] = head
-            header['keys'].append(key)
+            if "lengths" not in head:
+                head["lengths"] = tuple(map(nbytes, frames))
+            head["count"] = len(frames)
+            header["headers"][key] = head
+            header["keys"].append(key)
             out_frames.extend(frames)
 
         for i, frame in enumerate(out_frames):
             if type(frame) is memoryview and frame.strides != (1,):
                 try:
-                    frame = frame.cast('b')
+                    frame = frame.cast("b")
                 except TypeError:
                     frame = frame.tobytes()
                 out_frames[i] = frame
 
-        return [small_header, small_payload,
-                msgpack.dumps(header, use_bin_type=True)] + out_frames
+        return [
+            small_header,
+            small_payload,
+            msgpack.dumps(header, use_bin_type=True),
+        ] + out_frames
     except Exception:
         logger.critical("Failed to Serialize", exc_info=True)
         raise
@@ -101,14 +105,14 @@ def loads(frames, deserialize=True, deserializers=None):
             return msg
 
         header = frames.pop()
-        header = msgpack.loads(header, encoding='utf8', use_list=False)
-        keys = header['keys']
-        headers = header['headers']
-        bytestrings = set(header['bytestrings'])
+        header = msgpack.loads(header, use_list=False, **msgpack_opts)
+        keys = header["keys"]
+        headers = header["headers"]
+        bytestrings = set(header["bytestrings"])
 
         for key in keys:
             head = headers[key]
-            count = head['count']
+            count = head["count"]
             if count:
                 fs = frames[-count::][::-1]
                 del frames[-count:]
@@ -116,7 +120,7 @@ def loads(frames, deserialize=True, deserializers=None):
                 fs = []
 
             if deserialize or key in bytestrings:
-                if 'compression' in head:
+                if "compression" in head:
                     fs = decompress(head, fs)
                 fs = merge_frames(head, fs)
                 value = _deserialize(head, fs, deserializers=deserializers)
@@ -156,12 +160,12 @@ def dumps_msgpack(msg):
 
     fmt, payload = maybe_compress(payload)
     if fmt:
-        header['compression'] = fmt
+        header["compression"] = fmt
 
     if header:
         header_bytes = msgpack.dumps(header, use_bin_type=True)
     else:
-        header_bytes = b''
+        header_bytes = b""
 
     return [header_bytes, payload]
 
@@ -172,17 +176,20 @@ def loads_msgpack(header, payload):
     See Also:
         dumps_msgpack
     """
+    header = bytes(header)
     if header:
-        header = msgpack.loads(header, encoding='utf8', use_list=False)
+        header = msgpack.loads(header, use_list=False, **msgpack_opts)
     else:
         header = {}
 
-    if header.get('compression'):
+    if header.get("compression"):
         try:
-            decompress = compressions[header['compression']]['decompress']
+            decompress = compressions[header["compression"]]["decompress"]
             payload = decompress(payload)
         except KeyError:
-            raise ValueError("Data is compressed as %s but we don't have this"
-                             " installed" % str(header['compression']))
+            raise ValueError(
+                "Data is compressed as %s but we don't have this"
+                " installed" % str(header["compression"])
+            )
 
-    return msgpack.loads(payload, encoding='utf8', use_list=False)
+    return msgpack.loads(payload, use_list=False, **msgpack_opts)
