@@ -15,7 +15,6 @@ try:
 except ImportError:
     from toolz import topk
 
-BANDWIDTH = 100e6
 LATENCY = 10e-3
 log_2 = log(2)
 
@@ -134,7 +133,7 @@ class WorkStealing(SchedulerPlugin):
 
         nbytes = sum(dep.get_nbytes() for dep in ts.dependencies)
 
-        transfer_time = nbytes / BANDWIDTH + LATENCY
+        transfer_time = nbytes / self.scheduler.bandwidth + LATENCY
         split = ts.prefix
         if split in fast_tasks:
             return None, None
@@ -334,7 +333,7 @@ class WorkStealing(SchedulerPlugin):
                 saturated = [
                     ws
                     for ws in saturated
-                    if combined_occupancy(ws) > 0.2 and len(ws.processing) > ws.ncores
+                    if combined_occupancy(ws) > 0.2 and len(ws.processing) > ws.nthreads
                 ]
             elif len(s.saturated) < 20:
                 saturated = sorted(saturated, key=combined_occupancy, reverse=True)
@@ -380,7 +379,7 @@ class WorkStealing(SchedulerPlugin):
                             continue
                         if combined_occupancy(sat) < 0.2:
                             continue
-                        if len(sat.processing) <= sat.ncores:
+                        if len(sat.processing) <= sat.nthreads:
                             continue
 
                         i += 1
