@@ -9,7 +9,7 @@ from tornado import gen
 
 from distributed import Client, Variable, worker_client, Nanny, wait
 from distributed.metrics import time
-from distributed.utils_test import gen_cluster, inc, slow, div
+from distributed.utils_test import gen_cluster, inc, div
 from distributed.utils_test import client, cluster_fixture, loop  # noqa: F401
 
 
@@ -147,8 +147,8 @@ def test_timeout_get(c, s, a, b):
 
 
 @pytest.mark.skipif(sys.version_info[0] == 2, reason="Multi-client issues")
-@slow
-@gen_cluster(client=True, ncores=[("127.0.0.1", 2)] * 5, Worker=Nanny, timeout=None)
+@pytest.mark.slow
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 2)] * 5, Worker=Nanny, timeout=None)
 def test_race(c, s, *workers):
     NITERS = 50
 
