@@ -1,7 +1,9 @@
 from __future__ import print_function, division, absolute_import
 
 from concurrent.futures import ThreadPoolExecutor
+from datetime import timedelta
 import logging
+import multiprocessing
 from numbers import Number
 from operator import add
 import os
@@ -25,7 +27,7 @@ from distributed.core import rpc
 from distributed.client import wait
 from distributed.scheduler import Scheduler
 from distributed.metrics import time
-from distributed.worker import Worker, error_message, logger
+from distributed.worker import Worker, error_message, logger, parse_memory_limit
 from distributed.utils import tmpfile, format_bytes
 from distributed.utils_test import (
     inc,
@@ -33,7 +35,6 @@ from distributed.utils_test import (
     gen_cluster,
     div,
     dec,
-    slow,
     slowinc,
     gen_test,
     captured_logger,
@@ -49,12 +50,10 @@ from distributed.utils_test import (  # noqa: F401
 )
 
 
-def test_worker_ncores():
-    from distributed.worker import _ncores
-
+def test_worker_nthreads():
     w = Worker("127.0.0.1", 8019)
     try:
-        assert w.executor._max_workers == _ncores
+        assert w.executor._max_workers == multiprocessing.cpu_count()
     finally:
         shutil.rmtree(w.local_dir)
 
@@ -63,8 +62,8 @@ def test_worker_ncores():
 def test_str(s, a, b):
     assert a.address in str(a)
     assert a.address in repr(a)
-    assert str(a.ncores) in str(a)
-    assert str(a.ncores) in repr(a)
+    assert str(a.nthreads) in str(a)
+    assert str(a.nthreads) in repr(a)
     assert str(len(a.executing)) in repr(a)
 
 
@@ -73,7 +72,7 @@ def test_identity():
     ident = w.identity(None)
     assert "Worker" in ident["type"]
     assert ident["scheduler"] == "tcp://127.0.0.1:8019"
-    assert isinstance(ident["ncores"], int)
+    assert isinstance(ident["nthreads"], int)
     assert isinstance(ident["memory_limit"], Number)
 
 
@@ -147,7 +146,7 @@ def test_worker_bad_args(c, s, a, b):
     assert tuple(results) == (3, 7)
 
 
-@slow
+@pytest.mark.slow
 @gen_cluster()
 def dont_test_delete_data_with_missing_worker(c, a, b):
     bad = "127.0.0.1:9001"  # this worker doesn't exist
@@ -198,7 +197,7 @@ def test_upload_file(c, s, a, b):
 
 
 @pytest.mark.skip(reason="don't yet support uploading pyc files")
-@gen_cluster(client=True, ncores=[("127.0.0.1", 1)])
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)])
 def test_upload_file_pyc(c, s, w):
     with tmpfile() as dirname:
         os.mkdir(dirname)
@@ -304,8 +303,7 @@ def test_broadcast(s, a, b):
 
 @gen_test()
 def test_worker_with_port_zero():
-    s = Scheduler()
-    s.start(8007)
+    s = yield Scheduler(port=8007)
     w = yield Worker(s.address)
     assert isinstance(w.port, int)
     assert w.port > 1024
@@ -313,19 +311,24 @@ def test_worker_with_port_zero():
     yield w.close()
 
 
-@slow
-def test_worker_waits_for_center_to_come_up(loop):
+@pytest.mark.slow
+def test_worker_waits_for_scheduler(loop):
     @gen.coroutine
     def f():
-        w = yield Worker("127.0.0.1", 8007)
+        w = Worker("127.0.0.1", 8007)
+        try:
+            yield gen.with_timeout(timedelta(seconds=3), w)
+        except TimeoutError:
+            pass
+        else:
+            assert False
+        assert w.status not in ("closed", "running")
+        yield w.close(timeout=0.1)
 
-    try:
-        loop.run_sync(f, timeout=4)
-    except TimeoutError:
-        pass
+    loop.run_sync(f)
 
 
-@gen_cluster(client=True, ncores=[("127.0.0.1", 1)])
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)])
 def test_worker_task_data(c, s, w):
     x = delayed(2)
     xx = c.persist(x)
@@ -357,15 +360,16 @@ def test_gather(s, a, b):
         assert a.data["y"] == b.data["y"]
 
 
-def test_io_loop(loop):
-    s = Scheduler(loop=loop)
-    s.listen(0)
-    assert s.io_loop is loop
-    w = Worker(s.address, loop=loop)
-    assert w.io_loop is loop
+@pytest.mark.asyncio
+async def test_io_loop():
+    s = await Scheduler(port=0)
+    w = await Worker(s.address, loop=s.loop)
+    assert w.io_loop is s.loop
+    await s.close()
+    await w.close()
 
 
-@gen_cluster(client=True, ncores=[])
+@gen_cluster(client=True, nthreads=[])
 def test_spill_to_disk(c, s):
     np = pytest.importorskip("numpy")
     w = yield Worker(
@@ -382,17 +386,22 @@ def test_spill_to_disk(c, s):
     yield wait(y)
 
     assert set(w.data) == {x.key, y.key}
-    assert set(w.data.fast) == {x.key, y.key}
+    assert set(w.data.memory) == {x.key, y.key}
+    assert set(w.data.fast) == set(w.data.memory)
 
     z = c.submit(np.random.randint, 0, 255, size=500, dtype="u1", key="z")
     yield wait(z)
     assert set(w.data) == {x.key, y.key, z.key}
-    assert set(w.data.fast) == {y.key, z.key}
-    assert set(w.data.slow) == {x.key} or set(w.data.slow) == {x.key, y.key}
+    assert set(w.data.memory) == {y.key, z.key}
+    assert set(w.data.disk) == {x.key} or set(w.data.slow) == {x.key, y.key}
+    assert set(w.data.fast) == set(w.data.memory)
+    assert set(w.data.slow) == set(w.data.disk)
 
     yield x
-    assert set(w.data.fast) == {x.key, z.key}
-    assert set(w.data.slow) == {y.key} or set(w.data.slow) == {x.key, y.key}
+    assert set(w.data.memory) == {x.key, z.key}
+    assert set(w.data.disk) == {y.key} or set(w.data.slow) == {x.key, y.key}
+    assert set(w.data.fast) == set(w.data.memory)
+    assert set(w.data.slow) == set(w.data.disk)
     yield w.close()
 
 
@@ -431,10 +440,10 @@ def test_run_coroutine_dask_worker(c, s, a, b):
     assert response == {a.address: a.id, b.address: b.id}
 
 
-@gen_cluster(client=True, ncores=[])
+@gen_cluster(client=True, nthreads=[])
 def test_Executor(c, s):
     with ThreadPoolExecutor(2) as e:
-        w = Worker(s.ip, s.port, executor=e)
+        w = Worker(s.address, executor=e)
         assert w.executor is e
         w = yield w
 
@@ -452,7 +461,7 @@ def test_Executor(c, s):
 )
 @gen_cluster(
     client=True,
-    ncores=[("127.0.0.1", 1)],
+    nthreads=[("127.0.0.1", 1)],
     timeout=30,
     worker_kwargs={"memory_limit": 10e6},
 )
@@ -461,11 +470,11 @@ def test_spill_by_default(c, s, w):
     x = da.ones(int(10e6 * 0.7), chunks=1e6, dtype="u1")
     y = c.persist(x)
     yield wait(y)
-    assert len(w.data.slow)  # something is on disk
+    assert len(w.data.disk)  # something is on disk
     del x, y
 
 
-@gen_cluster(ncores=[("127.0.0.1", 1)], worker_kwargs={"reconnect": False})
+@gen_cluster(nthreads=[("127.0.0.1", 1)], worker_kwargs={"reconnect": False})
 def test_close_on_disconnect(s, w):
     yield s.close()
 
@@ -476,10 +485,10 @@ def test_close_on_disconnect(s, w):
 
 
 def test_memory_limit_auto():
-    a = Worker("127.0.0.1", 8099, ncores=1)
-    b = Worker("127.0.0.1", 8099, ncores=2)
-    c = Worker("127.0.0.1", 8099, ncores=100)
-    d = Worker("127.0.0.1", 8099, ncores=200)
+    a = Worker("127.0.0.1", 8099, nthreads=1)
+    b = Worker("127.0.0.1", 8099, nthreads=2)
+    c = Worker("127.0.0.1", 8099, nthreads=100)
+    d = Worker("127.0.0.1", 8099, nthreads=200)
 
     assert isinstance(a.memory_limit, Number)
     assert isinstance(b.memory_limit, Number)
@@ -575,7 +584,7 @@ def test_system_monitor(s, a, b):
 
 
 @gen_cluster(
-    client=True, ncores=[("127.0.0.1", 2, {"resources": {"A": 1}}), ("127.0.0.1", 1)]
+    client=True, nthreads=[("127.0.0.1", 2, {"resources": {"A": 1}}), ("127.0.0.1", 1)]
 )
 def test_restrictions(c, s, a, b):
     # Resource restrictions
@@ -605,7 +614,7 @@ def test_clean_nbytes(c, s, a, b):
     assert len(a.nbytes) + len(b.nbytes) == 1
 
 
-@gen_cluster(client=True, ncores=[("127.0.0.1", 1)] * 20)
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 20)
 def test_gather_many_small(c, s, a, *workers):
     a.total_out_connections = 2
     futures = yield c._scatter(list(range(100)))
@@ -626,7 +635,7 @@ def test_gather_many_small(c, s, a, *workers):
     assert a.comm_nbytes == 0
 
 
-@gen_cluster(client=True, ncores=[("127.0.0.1", 1)] * 3)
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 3)
 def test_multiple_transfers(c, s, w1, w2, w3):
     x = c.submit(inc, 1, workers=w1.address)
     y = c.submit(inc, 2, workers=w2.address)
@@ -639,7 +648,7 @@ def test_multiple_transfers(c, s, w1, w2, w3):
     assert len(transfers) == 2
 
 
-@gen_cluster(client=True, ncores=[("127.0.0.1", 1)] * 3)
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 3)
 def test_share_communication(c, s, w1, w2, w3):
     x = c.submit(mul, b"1", int(w3.target_message_size + 1), workers=w1.address)
     y = c.submit(mul, b"2", int(w3.target_message_size + 1), workers=w2.address)
@@ -722,14 +731,16 @@ def test_hold_onto_dependents(c, s, a, b):
     assert x.key in b.data
 
 
-@slow
-@gen_cluster(client=False, ncores=[])
+@pytest.mark.slow
+@gen_cluster(client=False, nthreads=[])
 def test_worker_death_timeout(s):
     with dask.config.set({"distributed.comm.timeouts.connect": "1s"}):
         yield s.close()
-        w = yield Worker(s.address, death_timeout=1)
+        w = Worker(s.address, death_timeout=1)
+
+    with pytest.raises(gen.TimeoutError):
+        yield w
 
-    yield gen.sleep(2)
     assert w.status == "closed"
 
 
@@ -746,7 +757,7 @@ def test_stop_doing_unnecessary_work(c, s, a, b):
         assert time() - start < 0.5
 
 
-@gen_cluster(client=True, ncores=[("127.0.0.1", 1)])
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)])
 def test_priorities(c, s, w):
     values = []
     for i in range(10):
@@ -832,7 +843,7 @@ def test_fail_write_to_disk(c, s, a, b):
 
 @pytest.mark.skip(reason="Our logic here is faulty")
 @gen_cluster(
-    ncores=[("127.0.0.1", 2)], client=True, worker_kwargs={"memory_limit": 10e9}
+    nthreads=[("127.0.0.1", 2)], client=True, worker_kwargs={"memory_limit": 10e9}
 )
 def test_fail_write_many_to_disk(c, s, a):
     a.validate = False
@@ -931,18 +942,13 @@ def test_get_client_coroutine_sync(client, s, a, b):
 
 @gen_cluster()
 def test_global_workers(s, a, b):
-    from distributed.worker import _global_workers
-
-    n = len(_global_workers)
-    w = _global_workers[-1]()
+    n = len(Worker._instances)
+    w = first(Worker._instances)
     assert w is a or w is b
-    yield a.close()
-    yield b.close()
-    assert len(_global_workers) == n - 2
 
 
 @pytest.mark.skipif(WINDOWS, reason="file descriptors")
-@gen_cluster(ncores=[])
+@gen_cluster(nthreads=[])
 def test_worker_fds(s):
     psutil = pytest.importorskip("psutil")
     yield gen.sleep(0.05)
@@ -964,51 +970,50 @@ def test_worker_fds(s):
         assert time() < start + 0.5
 
 
-@gen_cluster(ncores=[])
+@gen_cluster(nthreads=[])
 def test_service_hosts_match_worker(s):
     pytest.importorskip("bokeh")
-    from distributed.bokeh.worker import BokehWorker
+    from distributed.dashboard import BokehWorker
 
-    services = {("bokeh", ":0"): BokehWorker}
+    services = {("dashboard", ":0"): BokehWorker}
 
-    w = Worker(s.address, services={("bokeh", ":0"): BokehWorker})
+    w = Worker(s.address, services={("dashboard", ":0"): BokehWorker})
     yield w._start("tcp://0.0.0.0")
-    sock = first(w.services["bokeh"].server._http._sockets.values())
+    sock = first(w.services["dashboard"].server._http._sockets.values())
     assert sock.getsockname()[0] in ("::", "0.0.0.0")
     yield w.close()
 
-    w = Worker(s.address, services={("bokeh", ":0"): BokehWorker})
+    w = Worker(s.address, services={("dashboard", ":0"): BokehWorker})
     yield w._start("tcp://127.0.0.1")
-    sock = first(w.services["bokeh"].server._http._sockets.values())
+    sock = first(w.services["dashboard"].server._http._sockets.values())
     assert sock.getsockname()[0] in ("::", "0.0.0.0")
     yield w.close()
 
-    w = Worker(s.address, services={("bokeh", 0): BokehWorker})
+    w = Worker(s.address, services={("dashboard", 0): BokehWorker})
     yield w._start("tcp://127.0.0.1")
-    sock = first(w.services["bokeh"].server._http._sockets.values())
+    sock = first(w.services["dashboard"].server._http._sockets.values())
     assert sock.getsockname()[0] == "127.0.0.1"
     yield w.close()
 
 
-@gen_cluster(ncores=[])
+@gen_cluster(nthreads=[])
 def test_start_services(s):
     pytest.importorskip("bokeh")
-    from distributed.bokeh.worker import BokehWorker
+    from distributed.dashboard import BokehWorker
 
-    services = {("bokeh", ":1234"): BokehWorker}
+    services = {("dashboard", ":1234"): BokehWorker}
 
     w = Worker(s.address, services=services)
     yield w._start()
 
-    assert w.services["bokeh"].server.port == 1234
+    assert w.services["dashboard"].server.port == 1234
     yield w.close()
 
 
 @gen_test()
 def test_scheduler_file():
     with tmpfile() as fn:
-        s = Scheduler(scheduler_file=fn)
-        s.start(8009)
+        s = yield Scheduler(scheduler_file=fn, port=8009)
         w = yield Worker(scheduler_file=fn)
         assert set(s.workers) == {w.address}
         yield w.close()
@@ -1033,21 +1038,24 @@ def test_statistical_profiling(c, s, a, b):
     assert profile["count"]
 
 
+@pytest.mark.slow
 @nodebug
-@gen_cluster(client=True)
+@gen_cluster(client=True, timeout=20)
 def test_statistical_profiling_2(c, s, a, b):
     da = pytest.importorskip("dask.array")
-    for i in range(5):
+    while True:
         x = da.random.random(1000000, chunks=(10000,))
         y = (x + x * 2) - x.sum().persist()
         yield wait(y)
-    profile = a.get_profile()
-    assert profile["count"]
-    assert "sum" in str(profile) or "random" in str(profile)
+
+        profile = a.get_profile()
+        text = str(profile)
+        if profile["count"] and "sum" in text and "random" in text:
+            break
 
 
 @gen_cluster(
-    ncores=[("127.0.0.1", 1)],
+    nthreads=[("127.0.0.1", 1)],
     client=True,
     worker_kwargs={"memory_monitor_interval": 10},
 )
@@ -1071,14 +1079,14 @@ def test_robust_to_bad_sizeof_estimates(c, s, a):
     futures = c.map(f, [100e6] * 8, pure=False)
 
     start = time()
-    while not a.data.slow:
+    while not a.data.disk:
         yield gen.sleep(0.1)
         assert time() < start + 5
 
 
 @pytest.mark.slow
 @gen_cluster(
-    ncores=[("127.0.0.1", 2)],
+    nthreads=[("127.0.0.1", 2)],
     client=True,
     worker_kwargs={
         "memory_monitor_interval": 10,
@@ -1147,7 +1155,7 @@ def test_get_current_task(c, s, a, b):
     assert result.startswith("some_name")
 
 
-@gen_cluster(client=True, ncores=[("127.0.0.1", 1)] * 2)
+@gen_cluster(client=True, nthreads=[("127.0.0.1", 1)] * 2)
 def test_reschedule(c, s, a, b):
     s.extensions["stealing"]._pc.stop()
     a_address = a.address
@@ -1176,7 +1184,7 @@ def test_deque_handler():
     assert any(msg.msg == "foo456" for msg in deque_handler.deque)
 
 
-@gen_cluster(ncores=[], client=True)
+@gen_cluster(nthreads=[], client=True)
 def test_avoid_memory_monitor_if_zero_limit(c, s):
     worker = yield Worker(
         s.address, loop=s.loop, memory_limit=0, memory_monitor_interval=10
@@ -1194,7 +1202,7 @@ def test_avoid_memory_monitor_if_zero_limit(c, s):
 
 
 @gen_cluster(
-    ncores=[("127.0.0.1", 1)],
+    nthreads=[("127.0.0.1", 1)],
     config={
         "distributed.worker.memory.spill": False,
         "distributed.worker.memory.target": False,
@@ -1219,12 +1227,12 @@ def test_get_worker_name(client):
         assert time() < start + 10
 
 
-@gen_cluster(ncores=[("127.0.0.1", 1)], worker_kwargs={"memory_limit": "2e3 MB"})
+@gen_cluster(nthreads=[("127.0.0.1", 1)], worker_kwargs={"memory_limit": "2e3 MB"})
 def test_parse_memory_limit(s, w):
     assert w.memory_limit == 2e9
 
 
-@gen_cluster(ncores=[], client=True)
+@gen_cluster(nthreads=[], client=True)
 def test_scheduler_address_config(c, s):
     with dask.config.set({"scheduler-address": s.address}):
         worker = yield Worker(loop=s.loop)
@@ -1232,7 +1240,7 @@ def test_scheduler_address_config(c, s):
     yield worker.close()
 
 
-@slow
+@pytest.mark.slow
 @gen_cluster(client=True)
 def test_wait_for_outgoing(c, s, a, b):
     np = pytest.importorskip("numpy")
@@ -1253,7 +1261,9 @@ def test_wait_for_outgoing(c, s, a, b):
 @pytest.mark.skipif(
     not sys.platform.startswith("linux"), reason="Need 127.0.0.2 to mean localhost"
 )
-@gen_cluster(ncores=[("127.0.0.1", 1), ("127.0.0.1", 1), ("127.0.0.2", 1)], client=True)
+@gen_cluster(
+    nthreads=[("127.0.0.1", 1), ("127.0.0.1", 1), ("127.0.0.2", 1)], client=True
+)
 def test_prefer_gather_from_local_address(c, s, w1, w2, w3):
     x = yield c.scatter(123, workers=[w1.address, w3.address], broadcast=True)
 
@@ -1266,7 +1276,7 @@ def test_prefer_gather_from_local_address(c, s, w1, w2, w3):
 
 @gen_cluster(
     client=True,
-    ncores=[("127.0.0.1", 1)] * 20,
+    nthreads=[("127.0.0.1", 1)] * 20,
     timeout=30,
     config={"distributed.worker.connections.incoming": 1},
 )
@@ -1315,7 +1325,6 @@ def test_register_worker_callbacks(c, s, a, b):
         return os.getenv("MY_ENV_VALUE", None) == "WORKER_ENV_VALUE"
 
     # Nothing has been run yet
-    assert len(s.worker_setups) == 0
     result = yield c.run(test_import)
     assert list(result.values()) == [False] * 2
     result = yield c.run(test_startup2)
@@ -1330,7 +1339,6 @@ def test_register_worker_callbacks(c, s, a, b):
     # Add a preload function
     response = yield c.register_worker_callbacks(setup=mystartup)
     assert len(response) == 2
-    assert len(s.worker_setups) == 1
 
     # Check it has been ran on existing worker
     result = yield c.run(test_import)
@@ -1345,7 +1353,6 @@ def test_register_worker_callbacks(c, s, a, b):
     # Register another preload function
     response = yield c.register_worker_callbacks(setup=mystartup2)
     assert len(response) == 2
-    assert len(s.worker_setups) == 2
 
     # Check it has been run
     result = yield c.run(test_startup2)
@@ -1359,12 +1366,14 @@ def test_register_worker_callbacks(c, s, a, b):
     assert list(result.values()) == [True]
     yield worker.close()
 
-    # Final exception test
+
+@gen_cluster(client=True)
+def test_register_worker_callbacks_err(c, s, a, b):
     with pytest.raises(ZeroDivisionError):
         yield c.register_worker_callbacks(setup=lambda: 1 / 0)
 
 
-@gen_cluster(ncores=[])
+@gen_cluster(nthreads=[])
 def test_data_types(s):
     w = yield Worker(s.address, data=dict)
     assert isinstance(w.data, dict)
@@ -1384,3 +1393,49 @@ def test_data_types(s):
     assert w.data.x == 123
     assert w.data.y == 456
     yield w.close()
+
+
+@gen_cluster(nthreads=[])
+def test_local_dir(s):
+    with tmpfile() as fn:
+        with dask.config.set(temporary_directory=fn):
+            w = yield Worker(s.address)
+            assert w.local_dir.startswith(fn)
+            assert "dask-worker-space" in w.local_dir
+
+
+@pytest.mark.skipif(
+    not sys.platform.startswith("linux"), reason="Need 127.0.0.2 to mean localhost"
+)
+@gen_cluster(nthreads=[], client=True)
+def test_host_address(c, s):
+    w = yield Worker(s.address, host="127.0.0.2")
+    assert "127.0.0.2" in w.address
+    yield w.close()
+
+    n = yield Nanny(s.address, host="127.0.0.3")
+    assert "127.0.0.3" in n.address
+    assert "127.0.0.3" in n.worker_address
+    yield n.close()
+
+
+def test_resource_limit():
+    assert parse_memory_limit("250MiB", 1, total_cores=1) == 1024 * 1024 * 250
+
+    # get current limit
+    resource = pytest.importorskip("resource")
+    try:
+        hard_limit = resource.getrlimit(resource.RLIMIT_RSS)[1]
+    except OSError:
+        pytest.skip("resource could not get the RSS limit")
+    memory_limit = psutil.virtual_memory().total
+    if hard_limit > memory_limit or hard_limit < 0:
+        hard_limit = memory_limit
+
+    # decrease memory limit by one byte
+    new_limit = hard_limit - 1
+    try:
+        resource.setrlimit(resource.RLIMIT_RSS, (new_limit, new_limit))
+        assert parse_memory_limit(hard_limit, 1, total_cores=1) == new_limit
+    except OSError:
+        pytest.skip("resource could not set the RSS limit")
