@@ -23,7 +23,7 @@ import tempfile
 import threading
 import warnings
 import weakref
-
+import pkgutil
 import six
 import tblib.pickling_support
 
@@ -46,7 +46,7 @@ try:
 except ImportError:
     PollIOLoop = None  # dropped in tornado 6.0
 
-from .compatibility import Queue, PY3, PY2, get_thread_identity, unicode
+from .compatibility import PY3, PY2, get_thread_identity, unicode
 from .metrics import time
 
 
@@ -797,42 +797,6 @@ def truncate_exception(e, n=10000):
         return e
 
 
-if sys.version_info >= (3,):
-    # (re-)raising StopIteration is deprecated in 3.6+
-    exec(
-        """def queue_to_iterator(q):
-        while True:
-            result = q.get()
-            if isinstance(result, StopIteration):
-                return result.value
-            yield result
-        """
-    )
-else:
-    # Returning non-None from generator is a syntax error in 2.x
-    def queue_to_iterator(q):
-        while True:
-            result = q.get()
-            if isinstance(result, StopIteration):
-                raise result
-            yield result
-
-
-def _dump_to_queue(seq, q):
-    for item in seq:
-        q.put(item)
-
-
-def iterator_to_queue(seq, maxsize=0):
-    q = Queue(maxsize=maxsize)
-
-    t = threading.Thread(target=_dump_to_queue, args=(seq, q))
-    t.daemon = True
-    t.start()
-
-    return q
-
-
 def tokey(o):
     """ Convert an object to a string.
 
@@ -986,13 +950,13 @@ def tmpfile(extension=""):
     yield filename
 
     if os.path.exists(filename):
-        if os.path.isdir(filename):
-            shutil.rmtree(filename)
-        else:
-            try:
+        try:
+            if os.path.isdir(filename):
+                shutil.rmtree(filename)
+            else:
                 os.remove(filename)
-            except OSError:  # sometimes we can't remove a generated temp file
-                pass
+        except OSError:  # sometimes we can't remove a generated temp file
+            pass
 
 
 def ensure_bytes(s):
@@ -1102,14 +1066,11 @@ def import_file(path):
     if ext in (".egg", ".zip", ".pyz"):
         if path not in sys.path:
             sys.path.insert(0, path)
-        if ext == ".egg":
-            import pkg_resources
-
-            pkgs = pkg_resources.find_distributions(path)
-            for pkg in pkgs:
-                names_to_import.append(pkg.project_name)
-        elif ext in (".zip", ".pyz"):
-            names_to_import.append(name)
+        if sys.version_info >= (3, 6):
+            names = (mod_info.name for mod_info in pkgutil.iter_modules([path]))
+        else:
+            names = (mod_info[1] for mod_info in pkgutil.iter_modules([path]))
+        names_to_import.extend(names)
 
     loaded = []
     if not names_to_import:
@@ -1221,6 +1182,8 @@ def parse_bytes(s):
     >>> parse_bytes('MB')
     1000000
     """
+    if isinstance(s, (int, float)):
+        return int(s)
     s = s.replace(" ", "")
     if not s[0].isdigit():
         s = "1" + s
@@ -1279,6 +1242,8 @@ def parse_timedelta(s, default="seconds"):
     >>> parse_timedelta(timedelta(seconds=3))  # also supports timedeltas
     3
     """
+    if s is None:
+        return None
     if isinstance(s, timedelta):
         return s.total_seconds()
     if isinstance(s, Number):
