@@ -11,7 +11,6 @@ import itertools
 import logging
 import logging.config
 import os
-import psutil
 import re
 import shutil
 import signal
@@ -23,6 +22,7 @@ import textwrap
 import threading
 from time import sleep
 import uuid
+import warnings
 import weakref
 
 try:
@@ -40,11 +40,12 @@ from tornado.gen import TimeoutError
 from tornado.ioloop import IOLoop
 
 from .client import default_client, _global_clients, Client
-from .compatibility import PY3, Empty, WINDOWS, PY2
+from .compatibility import PY3, Empty, WINDOWS
 from .comm import Comm
 from .comm.utils import offload
 from .config import initialize_logging
 from .core import connect, rpc, CommClosedError
+from .deploy import SpecCluster
 from .metrics import time
 from .process import _cleanup_dangling
 from .proctitle import enable_proctitle_on_children
@@ -61,7 +62,8 @@ from .utils import (
     iscoroutinefunction,
     thread_state,
 )
-from .worker import Worker, TOTAL_MEMORY, _global_workers
+from .worker import Worker, TOTAL_MEMORY
+from .nanny import Nanny
 
 try:
     import dask.array  # register config
@@ -110,58 +112,41 @@ def invalid_python_script(tmpdir_factory):
 
 @gen.coroutine
 def cleanup_global_workers():
-    for w in _global_workers:
-        w = w()
-        w.close(report=False, executor_wait=False)
+    for worker in Worker._instances:
+        worker.close(report=False, executor_wait=False)
 
 
 @pytest.fixture
 def loop():
-    del _global_workers[:]
-    _global_clients.clear()
-    with pristine_loop() as loop:
-        # Monkey-patch IOLoop.start to wait for loop stop
-        orig_start = loop.start
-        is_stopped = threading.Event()
-        is_stopped.set()
+    with check_instances():
+        with pristine_loop() as loop:
+            # Monkey-patch IOLoop.start to wait for loop stop
+            orig_start = loop.start
+            is_stopped = threading.Event()
+            is_stopped.set()
 
-        def start():
-            is_stopped.clear()
+            def start():
+                is_stopped.clear()
+                try:
+                    orig_start()
+                finally:
+                    is_stopped.set()
+
+            loop.start = start
+
+            yield loop
+
+            # Stop the loop in case it's still running
             try:
-                orig_start()
-            finally:
-                is_stopped.set()
-
-        loop.start = start
-
-        yield loop
-
-        # Stop the loop in case it's still running
-        try:
-            sync(loop, cleanup_global_workers, callback_timeout=0.500)
-            loop.add_callback(loop.stop)
-        except RuntimeError as e:
-            if not re.match("IOLoop is clos(ed|ing)", str(e)):
-                raise
-        except gen.TimeoutError:
-            pass
-        else:
-            is_stopped.wait()
-    del _global_workers[:]
-
-    start = time()
-    while set(_global_clients):
-        sleep(0.1)
-        assert time() < start + 10
-
-    _cleanup_dangling()
-
-    if PY2:  # no forkserver, so no extra procs
-        for child in psutil.Process().children(recursive=True):
-            with ignoring(psutil.NoSuchProcess):
-                child.terminate()
-
-    _global_clients.clear()
+                sync(loop, cleanup_global_workers, callback_timeout=0.500)
+                loop.add_callback(loop.stop)
+            except RuntimeError as e:
+                if not re.match("IOLoop is clos(ed|ing)", str(e)):
+                    raise
+            except gen.TimeoutError:
+                pass
+            else:
+                is_stopped.wait()
 
 
 @pytest.fixture
@@ -411,7 +396,7 @@ def map_varying(itemslists):
     def apply(func, *args, **kwargs):
         return func(*args, **kwargs)
 
-    return apply, map(varying, itemslists)
+    return apply, list(map(varying, itemslists))
 
 
 @gen.coroutine
@@ -476,14 +461,14 @@ def readone(comm):
         raise gen.Return(msg)
 
 
-def run_scheduler(q, nputs, **kwargs):
+def run_scheduler(q, nputs, port=0, **kwargs):
     from distributed import Scheduler
 
     # On Python 2.7 and Unix, fork() is used to spawn child processes,
     # so avoid inheriting the parent's IO loop.
     with pristine_loop() as loop:
-        scheduler = Scheduler(validate=True, **kwargs)
-        done = scheduler.start("127.0.0.1")
+        scheduler = Scheduler(validate=True, host="127.0.0.1", port=port, **kwargs)
+        done = scheduler.start()
 
         for i in range(nputs):
             q.put(scheduler.address)
@@ -501,7 +486,7 @@ def run_worker(q, scheduler_q, **kwargs):
         with pristine_loop() as loop:
             scheduler_addr = scheduler_q.get()
             worker = Worker(scheduler_addr, validate=True, **kwargs)
-            loop.run_sync(lambda: worker._start(0))
+            loop.run_sync(lambda: worker._start())
             q.put(worker.address)
             try:
 
@@ -515,13 +500,11 @@ def run_worker(q, scheduler_q, **kwargs):
 
 
 def run_nanny(q, scheduler_q, **kwargs):
-    from distributed import Nanny
-
     with log_errors():
         with pristine_loop() as loop:
             scheduler_addr = scheduler_q.get()
             worker = Nanny(scheduler_addr, validate=True, **kwargs)
-            loop.run_sync(lambda: worker._start(0))
+            loop.run_sync(lambda: worker._start())
             q.put(worker.address)
             try:
                 loop.start()
@@ -636,143 +619,122 @@ def cluster(
     nworkers=2, nanny=False, worker_kwargs={}, active_rpc_timeout=1, scheduler_kwargs={}
 ):
     ws = weakref.WeakSet()
-
-    reset_config()
-    Comm._instances.clear()
-
-    for name, level in logging_levels.items():
-        logging.getLogger(name).setLevel(level)
-
     enable_proctitle_on_children()
 
-    with pristine_loop() as loop:
-        with check_active_rpc(loop, active_rpc_timeout):
-            if nanny:
-                _run_worker = run_nanny
-            else:
-                _run_worker = run_worker
+    with clean(timeout=active_rpc_timeout, threads=False) as loop:
+        if nanny:
+            _run_worker = run_nanny
+        else:
+            _run_worker = run_worker
 
-            # The scheduler queue will receive the scheduler's address
-            scheduler_q = mp_context.Queue()
+        # The scheduler queue will receive the scheduler's address
+        scheduler_q = mp_context.Queue()
 
-            # Launch scheduler
-            scheduler = mp_context.Process(
-                target=run_scheduler,
-                args=(scheduler_q, nworkers + 1),
-                kwargs=scheduler_kwargs,
+        # Launch scheduler
+        scheduler = mp_context.Process(
+            name="Dask cluster test: Scheduler",
+            target=run_scheduler,
+            args=(scheduler_q, nworkers + 1),
+            kwargs=scheduler_kwargs,
+        )
+        ws.add(scheduler)
+        scheduler.daemon = True
+        scheduler.start()
+
+        # Launch workers
+        workers = []
+        for i in range(nworkers):
+            q = mp_context.Queue()
+            fn = "_test_worker-%s" % uuid.uuid4()
+            kwargs = merge(
+                {"nthreads": 1, "local_dir": fn, "memory_limit": TOTAL_MEMORY},
+                worker_kwargs,
             )
-            ws.add(scheduler)
-            scheduler.daemon = True
-            scheduler.start()
-
-            # Launch workers
-            workers = []
-            for i in range(nworkers):
-                q = mp_context.Queue()
-                fn = "_test_worker-%s" % uuid.uuid4()
-                kwargs = merge(
-                    {"ncores": 1, "local_dir": fn, "memory_limit": TOTAL_MEMORY},
-                    worker_kwargs,
-                )
-                proc = mp_context.Process(
-                    target=_run_worker, args=(q, scheduler_q), kwargs=kwargs
-                )
-                ws.add(proc)
-                workers.append({"proc": proc, "queue": q, "dir": fn})
+            proc = mp_context.Process(
+                name="Dask cluster test: Worker",
+                target=_run_worker,
+                args=(q, scheduler_q),
+                kwargs=kwargs,
+            )
+            ws.add(proc)
+            workers.append({"proc": proc, "queue": q, "dir": fn})
 
+        for worker in workers:
+            worker["proc"].start()
+        try:
             for worker in workers:
-                worker["proc"].start()
+                worker["address"] = worker["queue"].get(timeout=5)
+        except Empty:
+            raise pytest.xfail.Exception("Worker failed to start in test")
+
+        saddr = scheduler_q.get()
+
+        start = time()
+        try:
             try:
-                for worker in workers:
-                    worker["address"] = worker["queue"].get(timeout=5)
-            except Empty:
-                raise pytest.xfail.Exception("Worker failed to start in test")
+                security = scheduler_kwargs["security"]
+                rpc_kwargs = {"connection_args": security.get_connection_args("client")}
+            except KeyError:
+                rpc_kwargs = {}
 
-            saddr = scheduler_q.get()
+            with rpc(saddr, **rpc_kwargs) as s:
+                while True:
+                    nthreads = loop.run_sync(s.ncores)
+                    if len(nthreads) == nworkers:
+                        break
+                    if time() - start > 5:
+                        raise Exception("Timeout on cluster creation")
 
-            start = time()
-            try:
-                try:
-                    security = scheduler_kwargs["security"]
-                    rpc_kwargs = {
-                        "connection_args": security.get_connection_args("client")
-                    }
-                except KeyError:
-                    rpc_kwargs = {}
+            # avoid sending processes down to function
+            yield {"address": saddr}, [
+                {"address": w["address"], "proc": weakref.ref(w["proc"])}
+                for w in workers
+            ]
+        finally:
+            logger.debug("Closing out test cluster")
 
-                with rpc(saddr, **rpc_kwargs) as s:
-                    while True:
-                        ncores = loop.run_sync(s.ncores)
-                        if len(ncores) == nworkers:
-                            break
-                        if time() - start > 5:
-                            raise Exception("Timeout on cluster creation")
-
-                # avoid sending processes down to function
-                yield {"address": saddr}, [
-                    {"address": w["address"], "proc": weakref.ref(w["proc"])}
-                    for w in workers
-                ]
-            finally:
-                logger.debug("Closing out test cluster")
-
-                loop.run_sync(
-                    lambda: disconnect_all(
-                        [w["address"] for w in workers],
-                        timeout=0.5,
-                        rpc_kwargs=rpc_kwargs,
-                    )
-                )
-                loop.run_sync(
-                    lambda: disconnect(saddr, timeout=0.5, rpc_kwargs=rpc_kwargs)
+            loop.run_sync(
+                lambda: disconnect_all(
+                    [w["address"] for w in workers], timeout=0.5, rpc_kwargs=rpc_kwargs
                 )
+            )
+            loop.run_sync(lambda: disconnect(saddr, timeout=0.5, rpc_kwargs=rpc_kwargs))
 
-                scheduler.terminate()
-                scheduler_q.close()
-                scheduler_q._reader.close()
-                scheduler_q._writer.close()
+            scheduler.terminate()
+            scheduler_q.close()
+            scheduler_q._reader.close()
+            scheduler_q._writer.close()
 
-                for w in workers:
-                    w["proc"].terminate()
-                    w["queue"].close()
-                    w["queue"]._reader.close()
-                    w["queue"]._writer.close()
+            for w in workers:
+                w["proc"].terminate()
+                w["queue"].close()
+                w["queue"]._reader.close()
+                w["queue"]._writer.close()
 
-                scheduler.join(2)
-                del scheduler
-                for proc in [w["proc"] for w in workers]:
-                    proc.join(timeout=2)
+            scheduler.join(2)
+            del scheduler
+            for proc in [w["proc"] for w in workers]:
+                proc.join(timeout=2)
 
-                with ignoring(UnboundLocalError):
-                    del worker, w, proc
-                del workers[:]
+            with ignoring(UnboundLocalError):
+                del worker, w, proc
+            del workers[:]
 
-                for fn in glob("_test_worker-*"):
-                    with ignoring(OSError):
-                        shutil.rmtree(fn)
+            for fn in glob("_test_worker-*"):
+                with ignoring(OSError):
+                    shutil.rmtree(fn)
 
-            try:
-                client = default_client()
-            except ValueError:
-                pass
-            else:
-                client.close()
+        try:
+            client = default_client()
+        except ValueError:
+            pass
+        else:
+            client.close()
 
     start = time()
-    while list(ws):
-        sleep(0.01)
-        assert time() < start + 1, "Workers still around after one second"
-
-    for i in range(5):
-        if all(c.closed() for c in Comm._instances):
-            break
-        else:
-            sleep(0.1)
-    else:
-        L = [c for c in Comm._instances if not c.closed()]
-        Comm._instances.clear()
-        print("Unclosed Comms", L)
-        # raise ValueError("Unclosed Comms", L)
+    while len(ws):
+        sleep(0.1)
+        assert time() < start + 3, ("Workers still around after two seconds", list(ws))
 
 
 @gen.coroutine
@@ -794,17 +756,6 @@ def disconnect_all(addresses, timeout=3, rpc_kwargs=None):
     yield [disconnect(addr, timeout, rpc_kwargs) for addr in addresses]
 
 
-def slow(func):
-    try:
-        if not pytest.config.getoption("--runslow"):
-            func = pytest.mark.skip("need --runslow option to run")(func)
-    except AttributeError:
-        # AttributeError: module 'pytest' has no attribute 'config'
-        pass
-
-    return nodebug(func)
-
-
 def gen_test(timeout=10):
     """ Coroutine test
 
@@ -815,15 +766,12 @@ def gen_test(timeout=10):
 
     def _(func):
         def test_func():
-            with pristine_loop() as loop:
+            with clean() as loop:
                 if iscoroutinefunction(func):
                     cor = func
                 else:
                     cor = gen.coroutine(func)
-                try:
-                    loop.run_sync(cor, timeout=timeout)
-                finally:
-                    loop.stop()
+                loop.run_sync(cor, timeout=timeout)
 
         return test_func
 
@@ -836,7 +784,7 @@ from .worker import Worker
 
 @gen.coroutine
 def start_cluster(
-    ncores,
+    nthreads,
     scheduler_addr,
     loop,
     security=None,
@@ -844,27 +792,30 @@ def start_cluster(
     scheduler_kwargs={},
     worker_kwargs={},
 ):
-    s = Scheduler(loop=loop, validate=True, security=security, **scheduler_kwargs)
+    s = Scheduler(
+        loop=loop, validate=True, security=security, port=0, **scheduler_kwargs
+    )
     done = s.start(scheduler_addr)
     workers = [
         Worker(
             s.address,
-            ncores=ncore[1],
+            nthreads=ncore[1],
             name=i,
             security=security,
             loop=loop,
             validate=True,
+            host=ncore[0],
             **(merge(worker_kwargs, ncore[2]) if len(ncore) > 2 else worker_kwargs)
         )
-        for i, ncore in enumerate(ncores)
+        for i, ncore in enumerate(nthreads)
     ]
     # for w in workers:
     #     w.rpc = workers[0].rpc
 
-    yield [w._start(ncore[0]) for ncore, w in zip(ncores, workers)]
+    yield workers
 
     start = time()
-    while len(s.workers) < len(ncores) or any(
+    while len(s.workers) < len(nthreads) or any(
         comm.comm is None for comm in s.stream_comms.values()
     ):
         yield gen.sleep(0.01)
@@ -890,7 +841,8 @@ def end_cluster(s, workers):
 
 
 def gen_cluster(
-    ncores=[("127.0.0.1", 1), ("127.0.0.1", 2)],
+    nthreads=[("127.0.0.1", 1), ("127.0.0.1", 2)],
+    ncores=None,
     scheduler="127.0.0.1",
     timeout=10,
     security=None,
@@ -915,6 +867,10 @@ def gen_cluster(
         start
         end
     """
+    if ncores is not None:
+        warnings.warn("ncores= has moved to nthreads=")
+        nthreads = ncores
+
     worker_kwargs = merge(
         {"memory_limit": TOTAL_MEMORY, "death_timeout": 5}, worker_kwargs
     )
@@ -924,143 +880,96 @@ def gen_cluster(
             func = gen.coroutine(func)
 
         def test_func():
-            del _global_workers[:]
-            _global_clients.clear()
-            Comm._instances.clear()
-            active_threads_start = set(threading._active)
-
-            reset_config()
-
-            dask.config.set({"distributed.comm.timeouts.connect": "5s"})
-            # Restore default logging levels
-            # XXX use pytest hooks/fixtures instead?
-            for name, level in logging_levels.items():
-                logging.getLogger(name).setLevel(level)
-
             result = None
             workers = []
+            with clean(threads=check_new_threads, timeout=active_rpc_timeout) as loop:
 
-            with pristine_loop() as loop:
-                with check_active_rpc(loop, active_rpc_timeout):
-
-                    @gen.coroutine
-                    def coro():
-                        with dask.config.set(config):
-                            s = False
-                            for i in range(5):
-                                try:
-                                    s, ws = yield start_cluster(
-                                        ncores,
-                                        scheduler,
-                                        loop,
-                                        security=security,
-                                        Worker=Worker,
-                                        scheduler_kwargs=scheduler_kwargs,
-                                        worker_kwargs=worker_kwargs,
-                                    )
-                                except Exception as e:
-                                    logger.error(
-                                        "Failed to start gen_cluster, retrying",
-                                        exc_info=True,
-                                    )
-                                else:
-                                    workers[:] = ws
-                                    args = [s] + workers
-                                    break
-                            if s is False:
-                                raise Exception("Could not start cluster")
-                            if client:
-                                c = yield Client(
-                                    s.address,
-                                    loop=loop,
+                @gen.coroutine
+                def coro():
+                    with dask.config.set(config):
+                        s = False
+                        for i in range(5):
+                            try:
+                                s, ws = yield start_cluster(
+                                    nthreads,
+                                    scheduler,
+                                    loop,
                                     security=security,
-                                    asynchronous=True,
-                                    **client_kwargs
+                                    Worker=Worker,
+                                    scheduler_kwargs=scheduler_kwargs,
+                                    worker_kwargs=worker_kwargs,
                                 )
-                                args = [c] + args
-                            try:
-                                future = func(*args)
-                                if timeout:
-                                    future = gen.with_timeout(
-                                        timedelta(seconds=timeout), future
-                                    )
-                                result = yield future
-                                if s.validate:
-                                    s.validate_state()
-                            finally:
-                                if client and c.status not in ("closing", "closed"):
-                                    yield c._close(fast=s.status == "closed")
-                                yield end_cluster(s, workers)
-                                yield gen.with_timeout(
-                                    timedelta(seconds=1), cleanup_global_workers()
+                            except Exception as e:
+                                logger.error(
+                                    "Failed to start gen_cluster, retrying",
+                                    exc_info=True,
                                 )
-
-                            try:
-                                c = yield default_client()
-                            except ValueError:
-                                pass
                             else:
-                                yield c._close(fast=True)
-
-                            for i in range(5):
-                                if all(c.closed() for c in Comm._instances):
-                                    break
-                                else:
-                                    yield gen.sleep(0.05)
-                            else:
-                                L = [c for c in Comm._instances if not c.closed()]
-                                Comm._instances.clear()
-                                # raise ValueError("Unclosed Comms", L)
-                                print("Unclosed Comms", L)
-
-                            raise gen.Return(result)
-
-                    result = loop.run_sync(
-                        coro, timeout=timeout * 2 if timeout else timeout
-                    )
-
-                for w in workers:
-                    if getattr(w, "data", None):
+                                workers[:] = ws
+                                args = [s] + workers
+                                break
+                        if s is False:
+                            raise Exception("Could not start cluster")
+                        if client:
+                            c = yield Client(
+                                s.address,
+                                loop=loop,
+                                security=security,
+                                asynchronous=True,
+                                **client_kwargs
+                            )
+                            args = [c] + args
                         try:
-                            w.data.clear()
-                        except EnvironmentError:
-                            # zict backends can fail if their storage directory
-                            # was already removed
+                            future = func(*args)
+                            if timeout:
+                                future = gen.with_timeout(
+                                    timedelta(seconds=timeout), future
+                                )
+                            result = yield future
+                            if s.validate:
+                                s.validate_state()
+                        finally:
+                            if client and c.status not in ("closing", "closed"):
+                                yield c._close(fast=s.status == "closed")
+                            yield end_cluster(s, workers)
+                            yield gen.with_timeout(
+                                timedelta(seconds=1), cleanup_global_workers()
+                            )
+
+                        try:
+                            c = yield default_client()
+                        except ValueError:
                             pass
-                        del w.data
-                DequeHandler.clear_all_instances()
-                for w in _global_workers:
-                    w = w()
-                    w.close(report=False, executor_wait=False)
-                    if w.status == "running":
-                        w.close()
-                del _global_workers[:]
+                        else:
+                            yield c._close(fast=True)
 
-            if PY3 and not WINDOWS and check_new_threads:
-                start = time()
-                while True:
-                    bad = [
-                        t
-                        for t, v in threading._active.items()
-                        if t not in active_threads_start
-                        and "Threaded" not in v.name
-                        and "watch message" not in v.name
-                        and "TCP-Executor" not in v.name
-                    ]
-                    if not bad:
-                        break
-                    else:
-                        sleep(0.01)
-                    if time() > start + 5:
-                        from distributed import profile
+                        for i in range(5):
+                            if all(c.closed() for c in Comm._instances):
+                                break
+                            else:
+                                yield gen.sleep(0.05)
+                        else:
+                            L = [c for c in Comm._instances if not c.closed()]
+                            Comm._instances.clear()
+                            # raise ValueError("Unclosed Comms", L)
+                            print("Unclosed Comms", L)
+
+                        raise gen.Return(result)
+
+                result = loop.run_sync(
+                    coro, timeout=timeout * 2 if timeout else timeout
+                )
+
+            for w in workers:
+                if getattr(w, "data", None):
+                    try:
+                        w.data.clear()
+                    except EnvironmentError:
+                        # zict backends can fail if their storage directory
+                        # was already removed
+                        pass
+                    del w.data
 
-                        tid = bad[0]
-                        thread = threading._active[tid]
-                        call_stacks = profile.call_stack(sys._current_frames()[tid])
-                        assert False, (thread, call_stacks)
-            _cleanup_dangling()
-            with ignoring(AttributeError):
-                del thread_state.on_event_loop_thread
             return result
 
         return test_func
@@ -1503,7 +1412,144 @@ def bump_rlimit(limit, desired):
 
 
 def gen_tls_cluster(**kwargs):
-    kwargs.setdefault("ncores", [("tls://127.0.0.1", 1), ("tls://127.0.0.1", 2)])
+    kwargs.setdefault("nthreads", [("tls://127.0.0.1", 1), ("tls://127.0.0.1", 2)])
     return gen_cluster(
         scheduler="tls://127.0.0.1", security=tls_only_security(), **kwargs
     )
+
+
+@contextmanager
+def save_sys_modules():
+    old_modules = sys.modules
+    old_path = sys.path
+    try:
+        yield
+    finally:
+        for i, elem in enumerate(sys.path):
+            if elem not in old_path:
+                del sys.path[i]
+        for elem in sys.modules.keys():
+            if elem not in old_modules:
+                del sys.modules[elem]
+
+
+@contextmanager
+def check_thread_leak():
+    active_threads_start = set(threading._active)
+
+    yield
+
+    start = time()
+    while True:
+        bad = [
+            t
+            for t, v in threading._active.items()
+            if t not in active_threads_start
+            and "Threaded" not in v.name
+            and "watch message" not in v.name
+            and "TCP-Executor" not in v.name
+        ]
+        if not bad:
+            break
+        else:
+            sleep(0.01)
+        if time() > start + 5:
+            from distributed import profile
+
+            tid = bad[0]
+            thread = threading._active[tid]
+            call_stacks = profile.call_stack(sys._current_frames()[tid])
+            assert False, (thread, call_stacks)
+
+
+@contextmanager
+def check_process_leak():
+    start_children = set(mp_context.active_children())
+
+    yield
+
+    for i in range(50):
+        if not set(mp_context.active_children()) - start_children:
+            break
+        else:
+            sleep(0.2)
+    else:
+        assert not mp_context.active_children()
+
+    _cleanup_dangling()
+
+
+@contextmanager
+def check_instances():
+    Client._instances.clear()
+    Worker._instances.clear()
+    Scheduler._instances.clear()
+    SpecCluster._instances.clear()
+    # assert all(n.status == "closed" for n in Nanny._instances), {
+    #     n: n.status for n in Nanny._instances
+    # }
+    Nanny._instances.clear()
+    _global_clients.clear()
+    Comm._instances.clear()
+
+    yield
+
+    start = time()
+    while set(_global_clients):
+        sleep(0.1)
+        assert time() < start + 10
+
+    _global_clients.clear()
+
+    for w in Worker._instances:
+        w.close(report=False, executor_wait=False)
+        if w.status == "running":
+            w.close()
+    Worker._instances.clear()
+
+    for i in range(5):
+        if all(c.closed() for c in Comm._instances):
+            break
+        else:
+            sleep(0.1)
+    else:
+        L = [c for c in Comm._instances if not c.closed()]
+        Comm._instances.clear()
+        print("Unclosed Comms", L)
+        # raise ValueError("Unclosed Comms", L)
+
+    assert all(n.status == "closed" or n.status == "init" for n in Nanny._instances), {
+        n: n.status for n in Nanny._instances
+    }
+
+    # assert not list(SpecCluster._instances)  # TODO
+    assert all(c.status == "closed" for c in SpecCluster._instances)
+    SpecCluster._instances.clear()
+
+    Nanny._instances.clear()
+    DequeHandler.clear_all_instances()
+
+
+@contextmanager
+def clean(threads=not WINDOWS, processes=True, instances=True, timeout=1):
+    @contextmanager
+    def null():
+        yield
+
+    with check_thread_leak() if threads else null():
+        with pristine_loop() as loop:
+            with check_process_leak() if processes else null():
+                with check_instances() if instances else null():
+                    with check_active_rpc(loop, timeout):
+                        reset_config()
+
+                        dask.config.set({"distributed.comm.timeouts.connect": "5s"})
+                        # Restore default logging levels
+                        # XXX use pytest hooks/fixtures instead?
+                        for name, level in logging_levels.items():
+                            logging.getLogger(name).setLevel(level)
+
+                        yield loop
+
+                        with ignoring(AttributeError):
+                            del thread_state.on_event_loop_thread
